/*
 * Copyright 1993, 1995, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* curve.c
 * This program draws a rotating NURBS curve.
 *
 *	Escape key	- exit the program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  animate( GLvoid );
GLvoid  visibility( GLint );
GLvoid  keyboard( GLubyte, GLint, GLint );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLUnurbsObj *theNurb;

static GLfloat angle = 0.0;

void
main(int argc, char *argv[])
{
	GLsizei 	width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width/4, height/4 ); 
	glutInitWindowSize( width/2, height/2 );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutIdleFunc( animate ); 
	glutVisibilityFunc( visibility ); 
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - draw a rotating NURBS curve\n"
		"Escape key		- exit the program\n\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	/* Create NURBS object */
	theNurb = gluNewNurbsRenderer();

	/* Set longest line segment to something lower than
	 * the default to get smooth looking curves.
	 */
	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 10.0);
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
animate( GLvoid )
{
	/* update the current angle */
	angle = fmodf( (angle + 5.0), 360.0 );

	/* Tell GLUT to redraw the scene */
	glutPostRedisplay();
}
        
GLvoid
visibility( int state )
{
	if (state == GLUT_VISIBLE) {
		glutIdleFunc( animate );
	} else {
		glutIdleFunc( NULL );
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -5.0 ); 
}

GLvoid
drawScene( GLvoid )
{
	GLfloat curvePts[4][3] = {
		{0.1, 0.5, 0.0}, {0.1, 0.9, 0.0}, 
		{0.9, 0.9, 0.0}, {0.9, 0.5, 0.0}
	};

    	GLfloat curveKnots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	glColor3f( 0.0, 1.0, 0.0 );
	glPushMatrix(); 
	
		glRotatef( angle, 0.0, 1.0, 0.0 );
		gluBeginCurve( theNurb );
			gluNurbsCurve( theNurb, 8, curveKnots, 3, 
				&curvePts[0][0], 4, GL_MAP1_VERTEX_3 );
		gluEndCurve( theNurb );

	glPopMatrix();

	glutSwapBuffers();
}

